package v1

import (
	"fmt"
	"git.sxidc.com/service-supports/dps-sdk"
	"git.sxidc.com/service-supports/dps-sdk/client"
	"github.com/stretchr/testify/assert"
	"testing"
)

var clientInstance client.Client

func initClient(t *testing.T, address string, databaseID string) {
	c, err := dps.NewClient(address, "v1", databaseID)
	if err != nil {
		t.Fatal(err)
	}

	clientInstance = c
}

func destroyClient(t *testing.T, databaseID string) {
	err := dps.DestroyClient("v1", databaseID)
	if err != nil {
		t.Fatal(err)
	}

	clientInstance = nil
}

type ToolKit struct {
	t *testing.T
}

func newToolKit(t *testing.T) *ToolKit {
	return &ToolKit{t: t}
}

func (toolKit *ToolKit) autoMigrate(req *client.AutoMigrateRequest) *ToolKit {
	err := clientInstance.AutoMigrate(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	return toolKit
}

func (toolKit *ToolKit) transaction(txFunc client.TransactionFunc) *ToolKit {
	err := clientInstance.Transaction(txFunc)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	return toolKit
}

func (toolKit *ToolKit) insert(req *client.InsertRequest) *ToolKit {
	statement, err := clientInstance.Insert(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) insertBatch(req *client.InsertBatchRequest) *ToolKit {
	statement, err := clientInstance.InsertBatch(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) delete(req *client.DeleteRequest) *ToolKit {
	statement, err := clientInstance.Delete(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) deleteWhere(req *client.DeleteWhereRequest) *ToolKit {
	statement, err := clientInstance.DeleteWhere(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) update(req *client.UpdateRequest) *ToolKit {
	statement, err := clientInstance.Update(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) updateWhere(req *client.UpdateWhereRequest) *ToolKit {
	statement, err := clientInstance.UpdateWhere(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) reply(req *client.ReplayRequest) *ToolKit {
	statement, err := clientInstance.Replay(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	return toolKit
}

func (toolKit *ToolKit) queryByWhereAndOrderBy(req *client.QueryByWhereAndOrderByRequest, retTableRows *[]client.TableRow, retTotalCount *int64) *ToolKit {
	statement, tableRows, totalCount, err := clientInstance.QueryByWhereAndOrderBy(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retTableRows != nil {
		*retTableRows = tableRows
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) commonQuery(req *client.CommonQueryRequest, retTableRows *[]client.TableRow, retTotalCount *int64) *ToolKit {
	statement, tableRows, totalCount, err := clientInstance.CommonQuery(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retTableRows != nil {
		*retTableRows = tableRows
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) queryOnlyByWhereAndOrderBy(req *client.QueryByWhereAndOrderByRequest, retTableRows *[]client.TableRow) *ToolKit {
	statement, tableRows, err := clientInstance.QueryOnlyByWhereAndOrderBy(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retTableRows != nil {
		*retTableRows = tableRows
	}

	return toolKit
}

func (toolKit *ToolKit) commonQueryOnly(req *client.CommonQueryRequest, retTableRows *[]client.TableRow) *ToolKit {
	statement, tableRows, err := clientInstance.CommonQueryOnly(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retTableRows != nil {
		*retTableRows = tableRows
	}

	return toolKit
}

func (toolKit *ToolKit) queryByKeys(req *client.QueryByKeysRequest, retTableRow *client.TableRow) *ToolKit {
	statement, tableRow, err := clientInstance.QueryByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retTableRow != nil {
		*retTableRow = *tableRow
	}

	return toolKit
}

func (toolKit *ToolKit) countWhere(req *client.CountWhereRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CountWhere(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) commonCount(req *client.CommonCountRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CommonCount(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) eventQueryByKeys(req *client.EventQueryByKeysRequest, retInfos *[]client.EventInfo, retTotalCount *int64) *ToolKit {
	statement, infos, totalCount, err := clientInstance.EventQueryByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = make([]client.EventInfo, 0)
		*retInfos = infos
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) commonEventQuery(req *client.CommonEventQueryRequest, retInfos *[]client.EventInfo, retTotalCount *int64) *ToolKit {
	statement, infos, totalCount, err := clientInstance.CommonEventQuery(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = make([]client.EventInfo, 0)
		*retInfos = infos
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) eventQueryOnlyByKeys(req *client.EventQueryByKeysRequest, retInfos *[]client.EventInfo) *ToolKit {
	statement, infos, err := clientInstance.EventQueryOnlyByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	return toolKit
}

func (toolKit *ToolKit) commonEventQueryOnly(req *client.CommonEventQueryRequest, retInfos *[]client.EventInfo) *ToolKit {
	statement, infos, err := clientInstance.CommonEventQueryOnly(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	return toolKit
}

func (toolKit *ToolKit) countEventByKeys(req *client.CountEventByKeysRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CountEventByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) commonCountEvent(req *client.CommonCountEventRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CommonCountEvent(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) eventHistoryQueryByKeys(req *client.EventQueryByKeysRequest, retInfos *[]client.EventInfo, retTotalCount *int64) *ToolKit {
	statement, infos, totalCount, err := clientInstance.EventHistoryQueryByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) commonEventHistoryQuery(req *client.CommonEventQueryRequest, retInfos *[]client.EventInfo, retTotalCount *int64) *ToolKit {
	statement, infos, totalCount, err := clientInstance.CommonEventHistoryQuery(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	if retTotalCount != nil {
		*retTotalCount = totalCount
	}

	return toolKit
}

func (toolKit *ToolKit) eventHistoryQueryOnlyByKeys(req *client.EventQueryByKeysRequest, retInfos *[]client.EventInfo) *ToolKit {
	statement, infos, err := clientInstance.EventHistoryQueryOnlyByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	return toolKit
}

func (toolKit *ToolKit) commonEventHistoryQueryOnly(req *client.CommonEventQueryRequest, retInfos *[]client.EventInfo) *ToolKit {
	statement, infos, err := clientInstance.CommonEventHistoryQueryOnly(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retInfos != nil {
		*retInfos = infos
	}

	return toolKit
}

func (toolKit *ToolKit) countEventHistoryByKeys(req *client.CountEventByKeysRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CountEventHistoryByKeys(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) commonCountEventHistory(req *client.CommonCountEventRequest, retCount *int64) *ToolKit {
	statement, count, err := clientInstance.CommonCountEventHistory(req)
	if err != nil {
		toolKit.t.Fatal(err)
	}

	fmt.Println(statement)

	if retCount != nil {
		*retCount = count
	}

	return toolKit
}

func (toolKit *ToolKit) assertEqual(expected interface{}, actual interface{}, msgAndArgs ...interface{}) *ToolKit {
	assert.Equal(toolKit.t, expected, actual, msgAndArgs)
	return toolKit
}

func (toolKit *ToolKit) assertNotEqual(expected interface{}, actual interface{}, msgAndArgs ...interface{}) *ToolKit {
	assert.NotEqual(toolKit.t, expected, actual, msgAndArgs)
	return toolKit
}

func (toolKit *ToolKit) assertNotEmpty(object interface{}, msgAndArgs ...interface{}) *ToolKit {
	assert.NotEmpty(toolKit.t, object, msgAndArgs)
	return toolKit
}
